#!/bin/sh

# To/Die/For =)
unset SHOULD_DIE
should_die() {
    RED='\033[0;31m'
    NC='\033[0m' # No Color
    echo -e "${RED}$1${NC}"
    SHOULD_DIE=1
}
die() {
    RED='\033[0;31m'
    NC='\033[0m' # No Color
    echo -e "${RED}$1${NC}"
    exit 1
}

##########################################################################################################################

# Set content type
CONTENT_TYPE="Content-Type: application/vnd.docker.distribution.manifest.v2+json"

# Check if env vars were set
test -z "${CI_SERVER_HOST}" && should_die "CI_SERVER_HOST is missing"
test -z "${CI_REGISTRY}" && should_die "CI_REGISTRY is missing"
test -z "${CI_REGISTRY_USER}" && should_die "CI_REGISTRY_USER is missing"
test -z "${CI_REGISTRY_PASSWORD}" && should_die "CI_REGISTRY_PASSWORD is missing"
test -z "${CI_PROJECT_PATH}" && should_die "CI_PROJECT_PATH is missing"
test -z "${CI_COMMIT_SHA}" && should_die "CI_COMMIT_SHA is missing"

if test -z "${CI_APPLICATION_TAG}" && test -z "${REGISTRY_TAG_OLD}"; then
    should_die "Neither CI_APPLICATION_TAG nor REGISTRY_TAG_OLD was set!"
fi
if test -n "${CI_APPLICATION_TAG}" && test -n "${REGISTRY_TAG_OLD}"; then
    should_die "Both CI_APPLICATION_TAG and REGISTRY_TAG_OLD were set!"
fi

##########################################################################################################################
test -n "${SHOULD_DIE}" && die "Wrong config!"
##########################################################################################################################

# DEBUG
if test -n "${RETAG_DEBUG}"; then
    env | sort -u
    set -x
fi

# TEST
if test -z "${CI_COMMIT_TAG}" && test -z "${REGISTRY_TAG_OLD}"; then
    REGISTRY_TAG_OLD="${CI_APPLICATION_TAG%%@*}"
    CUSTOM_IMAGENAME="${CI_COMMIT_REF_SLUG}"
elif test -n "${CI_COMMIT_TAG}" && test -z "${REGISTRY_TAG_OLD}"; then
    REGISTRY_TAG_OLD="${CI_COMMIT_TAG}"
fi

# Define default custom image name
test -z "${CUSTOM_IMAGENAME}" && IMAGE_PATH="${CI_PROJECT_PATH}" || IMAGE_PATH="${CI_PROJECT_PATH}/${CUSTOM_IMAGENAME}"

# Define default image new tag
test -z "${REGISTRY_TAG_NEW}" && REGISTRY_TAG_NEW=latest

##########################################################################################################################
TEMPFILE=$(mktemp)
##########################################################################################################################

# DEBUG
echo "Image: ${CI_REGISTRY}/${IMAGE_PATH}"
echo "Old tag: ${REGISTRY_TAG_OLD}"

# Fetch JWT token
TOKEN=$(curl -s \
    --user ${CI_REGISTRY_USER}:${CI_REGISTRY_PASSWORD} \
    -G ${GITLAB_SCHEMA:-https}://${CI_SERVER_HOST}/jwt/auth \
    -d service=container_registry \
    -d scope="repository:${IMAGE_PATH}:push,pull,delete" \
    | jq -r '.token')
test -n "${TOKEN}" || die "Couldn't get token"
BEARER="Authorization:Bearer $TOKEN"

# Set manifest root url
MANIFEST_URL=${REGISTRY_SCHEMA:-https}://${CI_REGISTRY}/v2/${IMAGE_PATH}/manifests

# Fetch image manifest
MANIFEST=$(curl -s \
    -H "${BEARER}" \
    -H "${CONTENT_TYPE}" \
    -G ${MANIFEST_URL}/${REGISTRY_TAG_OLD})
test -n "${MANIFEST}" || die "Couldn't get manifest"

# Push image manifest
RESPONCE=$(curl -s \
    -H "${BEARER}" \
    -H "${CONTENT_TYPE}" \
    -G ${MANIFEST_URL}/${REGISTRY_TAG_NEW} \
    -X PUT \
    -d "$MANIFEST")
test -z "${RESPONCE}" || die "Couldn't get responce"

echo "New tag: ${REGISTRY_TAG_NEW}"

##########################################################################################################################
rm -f "${TEMPFILE}"
